package ap.parser

import ap.terfor.conjunctions.Quantifier

/**
 * Class to turn <-> into conjunction and disjunctions
 */
object EquivExpander extends ContextAwareVisitor[Unit, IExpression] {

  import IExpression._
  
  def apply(f : IFormula) : IFormula =
    this.visit(f, Context()).asInstanceOf[IFormula]
  
  override def preVisit(t : IExpression, c : Context[Unit]) : PreVisitResult =
    t match {
      case IBinFormula(IBinJunctor.Eqv, f1, f2) =>
        if ((c.quans contains Quantifier.EX) ^ (c.polarity < 0))
          TryAgain((f1 & f2) | (!f1 & !f2), c)
        else
          TryAgain((f1 -> f2) & (f2 -> f1), c)
      case LeafFormula(_) | _ : ITerm =>
        // do not look into atoms or terms
        ShortCutResult(t)
      case _ =>
        super.preVisit(t, c)
    }

  def postVisit(t : IExpression, c : Context[Unit],
                subres : Seq[IExpression]) : IExpression =
    t update subres

}
