package ap.proof.goal;

import ap.terfor.{Formula, TermOrder}
import ap.terfor.conjunctions.{Conjunction, Quantifier}
import ap.terfor.preds.{Atom, PredConj, Predicate}
import ap.util.{Debug, Seqs, Logic}
import ap.proof.tree.{ProofTree, ProofTreeFactory}

object AddFactsTask {
  
  private val AC = Debug.AC_COMPLEX_FORMULAS_TASK

  /**
   * Return <code>true</code> if <code>f</code> is a formula that can be handled
   * by this task
   */
  def isCoveredFormula(f : Conjunction) : Boolean =
    !f.isFalse &&
    (f.isTrue || f.isLiteral ||
     (f.isNegatedConjunction && f.negatedConjs(0).negatedConjs.isEmpty)) && 
    f.quans.isEmpty

}

class AddFactsTask(_formula : Conjunction, _age : Int)
      extends FormulaTask(_formula, _age) {

  //////////////////////////////////////////////////////////////////////////////
  Debug.assertCtor(AddFactsTask.AC, !formula.isFalse)
  //////////////////////////////////////////////////////////////////////////////

  val priority : Int = age - 20000
  
  /**
   * Perform the actual task (whatever needs to be done with <code>formula</code>)
   */
  protected def doTask(goal : Goal, ptf : ProofTreeFactory) : ProofTree =
    if (formula.isTrue) {
      // then the goal can be closed immediately
      ptf.updateGoal(Conjunction.FALSE, goal)
    } else if (formula.isLiteral) {
      addFacts(Conjunction.negate(formula, goal.order), goal, ptf)
    } else {
      //////////////////////////////////////////////////////////////////////////
      Debug.assertInt(AddFactsTask.AC, formula.isNegatedConjunction)
      //////////////////////////////////////////////////////////////////////////

      val disj = formula.negatedConjs(0)
      //////////////////////////////////////////////////////////////////////////
      // possibly existing quantifiers should have been pulled out and
      // should not occur at this point
      Debug.assertInt(AddFactsTask.AC,
                      disj.quans.isEmpty && disj.negatedConjs.isEmpty)
      //////////////////////////////////////////////////////////////////////////
            
      // the literals of the negated conjunction can be added as facts to
      // the goal
      addFacts(disj, goal, ptf)
    }

  private def addFacts(newFacts : Conjunction,
                       goal : Goal,
                       ptf : ProofTreeFactory) : ProofTree = {
    val updatedFacts =
      Conjunction.conj(Array(goal.facts, newFacts), goal.order)
    val newTasks =
      if (newFacts.predConj.isTrue) List() else (LazyMatchTask addTask goal)
    ptf.updateGoal(updatedFacts, newTasks, goal)
  }
  
  /**
   * Create a new <code>FormulaTask</code> by updating the value of
   * <code>formula</code>
   */
  protected def updateFormula(f : Conjunction, goal : Goal) : FormulaTask =
    new AddFactsTask(f, age)

  /**
   * Return <code>true</code> if <code>f</code> is a formula that can be handled
   * by this task
   */
  protected def isCoveredFormula(f : Conjunction) : Boolean =
    AddFactsTask.isCoveredFormula(f)

  protected val name : String = "AddFacts"

}
