package ap.proof.goal;

import ap.terfor.{Formula, ConstantTerm}
import ap.terfor.conjunctions.{Conjunction, Quantifier}
import ap.terfor.substitutions.VariableSubst
import ap.util.{Debug, PlainRange}
import ap.proof.tree.{ProofTree, ProofTreeFactory}

object AllQuantifierTask {
  
  private val AC = Debug.AC_COMPLEX_FORMULAS_TASK
  
  /**
   * Return <code>true</code> if <code>f</code> is a formula that can be handled
   * by this task
   */
  def isCoveredFormula(f : Conjunction) : Boolean =
    f.quans.lastOption == Some(Quantifier.ALL)

}

class AllQuantifierTask(_formula : Conjunction, _age : Int)
      extends QuantifierTask(_formula, _age) {

  val priority : Int = -5000 + age

  protected val constantNameBase : String = "all_"
    
  protected def newEliminatedConstants(goal : Goal,
                                       newConstants : Iterable[ConstantTerm])
                : Set[ConstantTerm] = goal.eliminatedConstants ++ newConstants

  /**
   * Perform the actual task (whatever needs to be done with <code>formula</code>)
   */
  protected def doTask(goal : Goal, ptf : ProofTreeFactory) : ProofTree =
    instantiateWithConstants(false, goal, ptf)
  
  /**
   * Create a new <code>FormulaTask</code> by updating the value of
   * <code>formula</code>
   */
  protected def updateFormula(f : Conjunction, goal : Goal) : FormulaTask =
    new AllQuantifierTask(f, age)

  /**
   * Return <code>true</code> if <code>f</code> is a formula that can be handled
   * by this task
   */
  protected def isCoveredFormula(f : Conjunction) : Boolean =
    AllQuantifierTask.isCoveredFormula(f)

}
