package ap.proof.goal

import ap.proof.tree.{ProofTree, ProofTreeFactory}
import ap.terfor.conjunctions.Conjunction
import ap.util.Debug

private object MatchFunctions {
  def updateMatcher(goal : Goal,
                    ptf : ProofTreeFactory,
                    eager : Boolean) : ProofTree = {
    val order = goal.order
    val voc = goal.vocabulary

    val oldMatcher = goal.compoundFormulas quantifierClauses eager
    
    val (instances, newMatcher) =
      oldMatcher.updateFacts(goal.facts.predConj,
                             goal.mayAlias,
                             goal.reduceWithFacts,
                             (voc.constantFreedom.isShielded(_, voc.bindingContext)),
                             order)

    val newCF = goal.compoundFormulas.updateQuantifierClauses(eager, newMatcher)
    val newTasks = for (f <- instances; t <- goal.formulaTasks(f)) yield t
    
    ptf.updateGoal(newCF, newTasks, goal)
  }
}

////////////////////////////////////////////////////////////////////////////////

case object EagerMatchTask extends EagerTask {
  def apply(goal : Goal, ptf : ProofTreeFactory) : ProofTree =
    MatchFunctions.updateMatcher(goal, ptf, true)
}

////////////////////////////////////////////////////////////////////////////////

object LazyMatchTask {
  
  private val AC = Debug.AC_CLAUSE_MATCHER
  
  def addTask(goal : Goal) : Seq[PrioritisedTask] =
    if (goal.tasks.taskInfos.containsLazyMatchTask)
      List()
    else
      List(new LazyMatchTask (goal.age))
}

class LazyMatchTask(age : Int) extends PrioritisedTask {
  
  val priority : Int = -500 + age
  
  def updateTask(goal : Goal, factCollector : Conjunction => unit)
                : Seq[PrioritisedTask] = List(this)

  def apply(goal : Goal, ptf : ProofTreeFactory) : ProofTree = {
    ////////////////////////////////////////////////////////////////////////////
    Debug.assertPre(LazyMatchTask.AC,
                    !(goal.compoundFormulas.eagerQuantifiedClauses factsAreOutdated
                                                           goal.facts.predConj))
    ////////////////////////////////////////////////////////////////////////////
    MatchFunctions.updateMatcher(goal, ptf, false)
  }
  
  override def toString = "LazyMatchTask(" + priority + ")"
}