package ap.terfor.substitutions;

import ap.util.{Debug, Seqs}

import scala.collection.mutable.HashMap

object ConstantSubst {
  
  private val AC = Debug.AC_SUBSTITUTIONS
  
  def apply(c : ConstantTerm, replacement : Term, order : TermOrder) : ConstantSubst =
    new ConstantSubst(Map((c, replacement)), order)
  
  def apply(replacements : scala.collection.Map[ConstantTerm, Term],
            order : TermOrder) : ConstantSubst =
    new ConstantSubst(replacements, order)
  
}

/**
 * Replace a constant with an arbitrary term
 */
class ConstantSubst private
                    (private val replacements : scala.collection.Map[ConstantTerm, Term],
                     protected [substitutions] val order : TermOrder)
      extends SimpleSubstitution {

  //////////////////////////////////////////////////////////////////////////////
  Debug.assertCtor(ConstantSubst.AC, replacements.values forall (order isSortingOf _))
  //////////////////////////////////////////////////////////////////////////////     

  protected[substitutions] def passQuantifiers(num : Int) : Substitution =
    if (num == 0) {
      this
    } else {
      val shifter = new VariableShiftSubst(0, num, order)
      val newReplacements = new HashMap[ConstantTerm, Term]
      for ((c, t) <- replacements)
        newReplacements += (c -> shifter(t))
      // TODO: could be optimised by caching
      new ConstantSubst(newReplacements, order)
    }

  protected[substitutions] def applyToVariable(v : VariableTerm) : Term = v

  protected[substitutions] def applyToConstant(c : ConstantTerm) : Term =
    replacements.getOrElse(c, c)

  protected[substitutions] def isIdentityOn(t : TerFor) : Boolean =
    Seqs.disjointSeq(t.constants, replacements.keys)

  //////////////////////////////////////////////////////////////////////////////

  protected[substitutions] def after(that : ConstantSubst) : ConstantSubst = {
    val newReplacements = new HashMap[ConstantTerm, Term]
    newReplacements ++= this.replacements
    
    for ((c, t) <- that.replacements)
      newReplacements += (c -> this(t))
    
    new ConstantSubst(newReplacements, order)
  }
  
  //////////////////////////////////////////////////////////////////////////////

  def sortBy(newOrder : TermOrder) : ConstantSubst = {
    val newReplacements = new HashMap[ConstantTerm, Term]
    for ((c, t) <- replacements)
      newReplacements += (c -> (newOrder sort t))
    new ConstantSubst(newReplacements, newOrder)
  }

  //////////////////////////////////////////////////////////////////////////////

  override def toString : String =
    "[" + ((for ((c, t) <- replacements) yield ("" + c + " |-> " + t)) mkString ", ")

}
