package ap.terfor.substitutions;

import ap.terfor.linearcombination.LinearCombination
import ap.terfor.equations.{EquationConj, NegEquationConj}
import ap.terfor.inequalities.InEqConj
import ap.util.Debug

object SimpleSubstitution {

  private val AC = Debug.AC_SUBSTITUTIONS
  
}

/**
 * A substitution that works by simple replacement of constants or variables
 * with arbitrary terms
 */
trait SimpleSubstitution extends Substitution {

  protected[substitutions] def applyToVariable(v : VariableTerm) : Term

  protected[substitutions] def applyToConstant(c : ConstantTerm) : Term


  /**
   * Simple substitutions work by simple replacement
   */
  def apply(t : Term) : Term = t match {
    case t : VariableTerm => applyToVariable(t)
    case t : ConstantTerm => applyToConstant(t)
    case OneTerm => OneTerm
    case t : LinearCombination => apply(t)
  }
     
  def apply(lc : LinearCombination) : LinearCombination =
    idOrElse(lc,
             LinearCombination(for ((c, t) <- lc.elements)
                               yield (c, apply(t)),
                               order))

  protected[substitutions] def pseudoApply(lc : LinearCombination)
                                            : LinearCombination = apply(lc)

}
