package ap.util

object CmdlParser {

  /**
   * An boolean option of the kind "+opt" or "-opt"
   */
  object Opt {
    def unapply(arg : String) : Option[(String, Boolean)] =
      if (arg startsWith "+")
        Some(arg substring 1, true)
      else if (arg startsWith "-")
        Some((arg substring 1, false))
      else
        None
  }
  
  /**
   * An assignment of the kind "-var13=4" or "-xxx=hallo". The assigned value
   * is here only decoded to a string
   */  
  object ValueOpt {
    def unapply(arg : String) : Option[(String, String)] =
      if ((arg startsWith "-") && (arg contains "=")) {
        val i = arg indexOf "="
        Some(arg.substring(1, i), arg substring (i+1))
      } else {
        None
      }
  }

  /**
   * A value that represents an integer, like in "-var13=4"
   */  
  object IntVal {
    def unapply(arg : String) : Option[Int] =
      try {
        Some(arg.toInt) 
      } catch {
        case _ : NumberFormatException => None
      }
  }

  class UnknownArgumentException(arg : String)
      extends Exception("Unknown argument: " + arg)

}
